require 'redis'
require 'mongo_mapper'
require 'json'
require 'em-websocket'
require 'securerandom'
require 'sinatra/base'

$logger = true

module OpenPushed
	class Logger
		def self.log(message)
			if($logger)
				puts message
			end
		end

		def self.toggle
			if($logger)
				$logger = false
			else
				$logger = true
			end
		end
	end	
	class OpenPushedKeys
		include MongoMapper::Document

		key :secret, String
		key :channelid, String
		key :device, Boolean
	end
	class Server
		# Initialize mongomapper
		# Set the OpenPushedKeys class as an instance variable
		def initialize
			MongoMapper.connection = Mongo::Connection.new('localhost')
			MongoMapper.database = 'openpushed'
			
			@keys = OpenPushed::OpenPushedKeys
		end
		def run!
			$SOCKS = {}

			## Startup Redis Subscriber on channel notifications
			Thread.new do
				$redis = Redis.new

				$redis.subscribe("notifications") do|on|
					OpenPushed::Logger.log "Subcribed to channel"
					on.message do |channel,message|
						data = JSON.parse(message)
						$SOCKS[data["channel"]].each do |socket|
							socket.send(data["msg"])
						end
					end
				end
			end

			## Startup WebSockets Server
			Thread.new do
				EM.run {
				  EM::WebSocket.run(:host => "0.0.0.0", :port => 2803) do |ws|
				    ws.onopen { |handshake|
						OpenPushed::Logger.log "Opening connection"	 
				    }

				    ws.onclose { OpenPushed::Logger.log "Connection closed" }

				    ws.onmessage { |msg|
				     if(msg  == "0\n" || msg == "0")

				     else
				     	# Check if channel ID exists if not close we will close the socket immediately
				     	key = @keys.where(:channelid => msg).first
				     	if(key == nil)
				     		ws.close
				     	else
				     		OpenPushed::Logger.log "Client added to channel"
					     	if($SOCKS[msg] == nil) 
					      		$SOCKS[msg] = []
					  		end
					  		$SOCKS[msg] << ws
				  		end

				     end	
				    }

				  end
				}
			end

			Thread.new do
			end

			## Keep the proccess running so the Threads won't quit
			sleep
		end	
		def verbose(isit)
			$logger = isit
		end
	end

	class Migrate
		def self.run
			MongoMapper.connection = Mongo::Connection.new('localhost')
			MongoMapper.database = 'openpushed'

			key = OpenPushed::OpenPushedKeys.new(:secret => SecureRandom.hex,:channelid => SecureRandom.hex,:device => false);
			key.save!
		end
	end
	class WebAPI < Sinatra::Base
		# Setup port, mongomapper connections and redis
		configure do 
			set :port,2804

			MongoMapper.connection = Mongo::Connection.new('localhost')
			MongoMapper.database = 'openpushed'
			
			$redis = Redis.new
			$keys = OpenPushed::OpenPushedKeys
		end
		# Route to broadcast to all subscribed devices
		post '/api/push/broadcast' do
			key = $keys.where(:secret => params[:secret],:channelid => params[:channelid]).first
			if(key == nil)
				{:error => "invalid keys"}.to_json
			else
				$redis.publish "notifications", {:channel => params[:channelid], :msg => params[:message]}.to_json
				{:status => "ok"}.to_json
			end
		end
	end
end